﻿using System.ComponentModel;

namespace TestProject;

/// <summary>
/// 호출 동기화 확장
/// </summary>
public static class SynchronizeInvokeExtension
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Static
    //////////////////////////////////////////////////////////////////////////////// Public

    #region 필요시 호출하기 - InvokeIfRequired<TTarget>(targetObject, action)

    /// <summary>
    /// 필요시 호출하기
    /// </summary>
    /// <typeparam name="TTarget">타겟 객체 타입</typeparam>
    /// <param name="targetObject">타겟 객체</param>
    /// <param name="action">액션</param>
    public static void InvokeIfRequired<TTarget>(this TTarget targetObject, Action<TTarget> action) where TTarget : ISynchronizeInvoke
    {
        if(targetObject.InvokeRequired)
        {
            targetObject.Invoke(action, new object[] { targetObject });
        }
        else
        {
            action(targetObject);
        }
    }

    #endregion
    #region 필요시 호출하기 - InvokeIfRequired<TTarget, TResult>(targetObject, function)

    /// <summary>
    /// 필요시 호출하기
    /// </summary>
    /// <typeparam name="TTarget">타겟 객체 타입</typeparam>
    /// <typeparam name="TResult">결과 타입</typeparam>
    /// <param name="targetObject">타겟 객체</param>
    /// <param name="function">함수</param>
    /// <returns>처리 결과</returns>
    public static TResult InvokeIfRequired<TTarget, TResult>(this TTarget targetObject, Func<TTarget, TResult> function) where TTarget : ISynchronizeInvoke
    {
        return targetObject.InvokeRequired ? (TResult)targetObject.Invoke(function, new object[] { targetObject }) : function(targetObject);
    }

    #endregion
}